__author__ = ["Aleksandra Apolinarska"]
__copyright__ = "Copyright (c) 2023-2024 ETH Zurich, Gramazio Kohler Research"
__license__ = "MIT License"
__email__ = "apolinarska@arch.ethz.ch"
__version__ = "0.10.4"
